package org.springframework.boot.loader;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.springframework.boot.loader.ClassPathIndexFile;
import org.springframework.boot.loader.Launcher;
import org.springframework.boot.loader.archive.Archive;

public abstract class ExecutableArchiveLauncher extends Launcher {
  private static final String START_CLASS_ATTRIBUTE = "Start-Class";
  
  protected static final String BOOT_CLASSPATH_INDEX_ATTRIBUTE = "Spring-Boot-Classpath-Index";
  
  protected static final String DEFAULT_CLASSPATH_INDEX_FILE_NAME = "classpath.idx";
  
  private final Archive archive;
  
  private final ClassPathIndexFile classPathIndex;
  
  public ExecutableArchiveLauncher() {
    try {
      this.archive = createArchive();
      this.classPathIndex = getClassPathIndex(this.archive);
    } catch (Exception ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  protected ExecutableArchiveLauncher(Archive archive) {
    try {
      this.archive = archive;
      this.classPathIndex = getClassPathIndex(this.archive);
    } catch (Exception ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  protected ClassPathIndexFile getClassPathIndex(Archive archive) throws IOException {
    if (archive instanceof org.springframework.boot.loader.archive.ExplodedArchive) {
      String location = getClassPathIndexFileLocation(archive);
      return ClassPathIndexFile.loadIfPossible(archive.getUrl(), location);
    } 
    return null;
  }
  
  private String getClassPathIndexFileLocation(Archive archive) throws IOException {
    Manifest manifest = archive.getManifest();
    Attributes attributes = (manifest != null) ? manifest.getMainAttributes() : null;
    String location = (attributes != null) ? attributes.getValue("Spring-Boot-Classpath-Index") : null;
    return (location != null) ? location : (getArchiveEntryPathPrefix() + "classpath.idx");
  }
  
  protected String getMainClass() throws Exception {
    Manifest manifest = this.archive.getManifest();
    String mainClass = null;
    if (manifest != null)
      mainClass = manifest.getMainAttributes().getValue("Start-Class"); 
    if (mainClass == null)
      throw new IllegalStateException("No 'Start-Class' manifest entry specified in " + this); 
    return mainClass;
  }
  
  protected ClassLoader createClassLoader(Iterator<Archive> archives) throws Exception {
    List<URL> urls = new ArrayList<>(guessClassPathSize());
    while (archives.hasNext())
      urls.add(((Archive)archives.next()).getUrl()); 
    if (this.classPathIndex != null)
      urls.addAll(this.classPathIndex.getUrls()); 
    return createClassLoader(urls.<URL>toArray(new URL[0]));
  }
  
  private int guessClassPathSize() {
    if (this.classPathIndex != null)
      return this.classPathIndex.size() + 10; 
    return 50;
  }
  
  protected Iterator<Archive> getClassPathArchivesIterator() throws Exception {
    Archive.EntryFilter searchFilter = this::isSearchCandidate;
    Iterator<Archive> archives = this.archive.getNestedArchives(searchFilter, entry -> 
        (isNestedArchive(entry) && !isEntryIndexed(entry)));
    if (isPostProcessingClassPathArchives())
      archives = applyClassPathArchivePostProcessing(archives); 
    return archives;
  }
  
  private boolean isEntryIndexed(Archive.Entry entry) {
    if (this.classPathIndex != null)
      return this.classPathIndex.containsEntry(entry.getName()); 
    return false;
  }
  
  private Iterator<Archive> applyClassPathArchivePostProcessing(Iterator<Archive> archives) throws Exception {
    List<Archive> list = new ArrayList<>();
    while (archives.hasNext())
      list.add(archives.next()); 
    postProcessClassPathArchives(list);
    return list.iterator();
  }
  
  protected boolean isSearchCandidate(Archive.Entry entry) {
    if (getArchiveEntryPathPrefix() == null)
      return true; 
    return entry.getName().startsWith(getArchiveEntryPathPrefix());
  }
  
  protected boolean isPostProcessingClassPathArchives() {
    return true;
  }
  
  protected void postProcessClassPathArchives(List<Archive> archives) throws Exception {}
  
  protected String getArchiveEntryPathPrefix() {
    return null;
  }
  
  protected boolean isExploded() {
    return this.archive.isExploded();
  }
  
  protected final Archive getArchive() {
    return this.archive;
  }
  
  protected abstract boolean isNestedArchive(Archive.Entry paramEntry);
}
